@@ -1,3 +1,5 @@
+from ._tdigest_mixin import TDigestCommandsMixin
+from ._timeseries_mixin import TimeSeriesCommandsMixin
 from ._topk_mixin import TopkCommandsMixin  # noqa: F401
 
 try:
@@ -8,10 +10,10 @@ except ImportError as e:
     if e.name == "fakeredis.stack._json_mixin":
         raise e
 
-
     class JSONCommandsMixin:  # type: ignore # noqa: E303
         pass
 
+
 try:
     import probables  # noqa: F401
 
@@ -22,22 +24,23 @@ except ImportError as e:
     if e.name == "fakeredis.stack._bf_mixin" or e.name == "fakeredis.stack._cf_mixin":
         raise e
 
-
     class BFCommandsMixin:  # type: ignore # noqa: E303
         pass
 
-
     class CFCommandsMixin:  # type: ignore # noqa: E303
         pass
 
-
     class CMSCommandsMixin:  # type: ignore # noqa: E303
         pass
 
+
 __all__ = [
     "TopkCommandsMixin",
     "JSONCommandsMixin",
     "JSONObject",
     "BFCommandsMixin",
     "CFCommandsMixin",
-    "CMSCommandsMixin", ]
+    "CMSCommandsMixin",
+    "TDigestCommandsMixin",
+    "TimeSeriesCommandsMixin",
+]
